@@ -1,4 +1,4 @@
-﻿#ifdef WINDOWS
+#ifdef WINDOWS
 #include <io.h>
 #include <fcntl.h>
 #endif
@@ -10,8 +10,11 @@
 #include "../../../config.h"
 
 #include <gtk/gtk.h>
-
-#include <gdk/gdkkeysyms.h>
+#if GTKVERSION==2
+  #include <gdk/gdkkeysyms.h>
+#else
+  #include <gdk/gdk.h>
+#endif
 
 #include <cairo.h>
 #include <cairo-ps.h>
@@ -36,10 +39,11 @@ static DCL_REAL rwxold, rwyold;
 #define PAD    2.0           /* padding for workstation window */
 #define LWDATR TRUE          /* line width  capability */
 #define LCLATR TRUE          /* line color  capability */
-#define LTNATR TRUE          /* hard fill   capability */
+#define LTNATR FALSE         /* hardfill image   capability */
 #define LIMATR TRUE          /* bit image   capability */
 #define LPTATR FALSE         /* mouse point capability */
 
+
 #define MAXWDI 9             /* maximum number of line index */
 #define MAXCLI 255           /* maximum number of line color */
 #define MAXBMP 300           /* maximum number of bitmaps */
@@ -90,8 +94,6 @@ static int laltz = FALSE;
 
 static int pactive;
 
-static GdkGC *gc;
-static GdkGC *gct;
 static GtkWidget *window = NULL;
 static GtkWidget *drawing_area = NULL;
 
@@ -136,9 +138,6 @@ static double dlnscale;
 static double offx = 0.0;
 static double offy = 0.0;
 
-/* Tone Color (Full Color)*/
-static double rt2,gt2,bt2;
-
 /* For BITMAP Image */
 static unsigned char * pixels;
 static int rows;
@@ -155,6 +154,9 @@ static char fontlist[255] = { 41, 42, 43, 44, 55, 66, 77, 85, 95 };
 //  Full Color index */
 static double rt2,gt2,bt2,rl2,gl2,bl2;
 static int ifcidx=998;
+//Background color index
+static int ibgcli=999;
+
 
 /* For using system font. some internal variables of SGPACK*/
 static int sg_lfprop, sg_lcntl, sg_isup, sg_isub, sg_irst;
@@ -201,9 +203,13 @@ int getparmnumber(char pmname[],int tp){
       }
       break;
     }
+
+
+
   return num;
 }
 
+
 #ifndef WINDOWS
 void zglset_(char pmname[],int *pl,DCL_INT* value)
 #else
@@ -290,26 +296,46 @@ void ZGIINT(DCL_INT *iwx, DCL_INT *iwy, DCL_REAL *wx, DCL_REAL *wy)
 
 /*------------------------- GDK 設定 ------------------------*/
 
+#if GTKVERSION==2
 void zgupdate(int flag)
+#else
+void zgupdate(int flag,cairo_t* cr )
+#endif
 {
-    GdkRectangle update_rect;
-
-    if (idev_type==IWS_DISP){
-      update_rect.x = irxb-1;
-      update_rect.y = iryb-1;
-      update_rect.width  = irxt-irxb+1;
-      update_rect.height = iryt-iryb+1;
-
-      if (irxb == 0 && irxt == 0 && iryb == 0 && iryt == 0 && flag == 0){
-        update_rect.x = 0;
-        update_rect.y = 0;
-        update_rect.width  = wsxwd;
-        update_rect.height = wsywd;
-      }
-
+#if GTKVERSION==2
+  GdkRectangle update_rect;
+  if (idev_type==IWS_DISP){
+    update_rect.x = irxb-1;
+    update_rect.y = iryb-1;
+    update_rect.width  = irxt-irxb+1;
+    update_rect.height = iryt-iryb+1;
+    if (irxb == 0 && irxt == 0 && iryb == 0 && iryt == 0 && flag == 0){
+      update_rect.x = 0;
+      update_rect.y = 0;
+      update_rect.width  = wsxwd;
+      update_rect.height = wsywd;
+    }
+#else
+  GtkAllocation update_rect;
+  if (idev_type==IWS_DISP){
+    update_rect.x = irxb-1;
+    update_rect.y = iryb-1;
+    update_rect.width  = irxt-irxb+1;
+    update_rect.height = iryt-iryb+1;
+    if (irxb == 0 && irxt == 0 && iryb == 0 && iryt == 0 && flag == 0){
+      update_rect.x = 0;
+      update_rect.y = 0;
+      update_rect.width  = wsxwd-1;
+      update_rect.height = wsywd-1;
+    }
+#endif
 
+#if GTKVERSION==2
       gtk_widget_draw(drawing_area, &update_rect);
-
+#else
+      gtk_widget_size_allocate(drawing_area, &update_rect);
+      gtk_widget_draw(drawing_area, cr);
+#endif
       irxt = 0;
       irxb = 0;
       iryt = 0;
@@ -328,18 +354,29 @@ void zguprect(int ix,int iy){
 
 
 /*-----------------------------------------------*/
+#if GTKVERSION==2
 gint zg_expose_event(GtkWidget *widget, GdkEventExpose *event)
 {
 static cairo_t *cre;
   if (idev_type==IWS_DISP && lpage){
-//    cre = gdk_cairo_create (drawing_area->window);
-    cre = gdk_cairo_create (widget->window);
+    cre = gdk_cairo_create(widget->window);
     cairo_set_source_surface(cre,csr,0,0);
     cairo_paint(cre);
     cairo_destroy(cre);
   }
   return FALSE;
 }
+#else
+gboolean zg_draw_event(GtkWidget *widget, cairo_t *cr)
+{
+  if (idev_type==IWS_DISP && lpage){
+//    cre = gdk_cairo_create(gtk_widget_get_parent_window(widget));
+    cairo_set_source_surface(cr,csr,0,0);
+    cairo_paint(cr);
+  }
+  return(FALSE);
+}
+#endif
 
 gint zg_key_press_event(GtkWidget *widget, GdkEventKey *event)
 {
@@ -350,8 +387,13 @@ gint zg_key_press_event(GtkWidget *widget, GdkEventKey *event)
 
   case DCLS:
     switch (event->keyval) {
+#if GTKVERSION==2
     case GDK_space:
     case GDK_Return:
+#else
+    case GDK_KEY_space:
+    case GDK_KEY_Return:
+#endif
       next = TRUE;
       break;
     default:
@@ -361,23 +403,44 @@ gint zg_key_press_event(GtkWidget *widget, GdkEventKey *event)
 
   case PCLS:
     switch (event->keyval) {
+#if GTKVERSION==2
     case GDK_w:
+#else
+    case GDK_KEY_w:
+#endif
       wait_np = TRUE;
       break;
+#if GTKVERSION==2
     case GDK_Return:
     case GDK_space:
+#else
+    case GDK_KEY_Return:
+    case GDK_KEY_space:
+#endif
       next = TRUE;
       break;
+#if GTKVERSION==2
     case GDK_d:
+#else
+    case GDK_KEY_d:
+#endif
       next = TRUE;
       dumpz = TRUE;
       break;
+#if GTKVERSION==2
     case GDK_s:
+#else
+    case GDK_KEY_s:
+#endif
       next = TRUE;
       wait_np = FALSE;
       break;
+#if GTKVERSION==2
     case GDK_q:
       gtk_exit(0);
+#else
+    case GDK_KEY_q:
+#endif
       exit(0);
     default:
       next = FALSE;
@@ -407,7 +470,9 @@ gint zg_button_press_event(GtkWidget *widget, GdkEventButton *event)
 
 gint zg_destroy_event(GtkWidget *widget, GdkEventButton *event)
 {
-  gtk_exit(0);
+#if GTKVERSION==2
+      gtk_exit(0);
+#endif
   exit(0);
   return TRUE;
 }
@@ -424,37 +489,65 @@ static GtkWidget *ZGGDRW(DCL_INT *width, DCL_INT *height)
 
   gtk_init(0,NULL);
 
-  drw = gtk_drawing_area_new();
-  gtk_drawing_area_size(GTK_DRAWING_AREA (drw), wsxwd, wsywd);
-  gtk_widget_show(drw);
+#if GTKVERSION==2
+  drw=gtk_drawing_area_new();
+#else
+  drw = (GtkWidget*)gtk_drawing_area_new();
+#endif
 
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (window), xtitle);
-  gtk_container_add(GTK_CONTAINER (window), drw);
 
-  if (!wnd)
+#if GTKVERSION==2
+  gtk_drawing_area_size(GTK_DRAWING_AREA (drw), wsxwd, wsywd);
+#else
+  gtk_widget_set_size_request((GtkWidget*)drw, wsxwd, wsywd);
+#endif
+
+  gtk_widget_show(drw);
+  gtk_window_set_title (GTK_WINDOW (window), xtitle);
+
+#if GTKVERSION==2
+  gtk_container_add(GTK_CONTAINER (window), drw);
+#else
+  gtk_container_add(GTK_CONTAINER(window),drw);
+#endif
+  if (!wnd){
     gtk_widget_realize(drw);
-  else {
+  }else {
     if (key){
+#if GTKVERSION==2
       gtk_widget_set_events(window, GDK_BUTTON_PRESS_MASK|GDK_KEY_PRESS_MASK);
       gtk_signal_connect(GTK_OBJECT (window), "key_press_event",
 			 (GtkSignalFunc) zg_key_press_event, NULL);
-    } else
+
+#else
+      gtk_widget_set_events(window, GDK_BUTTON_PRESS_MASK|GDK_KEY_PRESS_MASK);
+      g_signal_connect(G_OBJECT(window), "button_press_event",
+                          G_CALLBACK(zg_button_press_event), NULL);
+#endif
+    }
+      gtk_widget_set_events(window, GDK_BUTTON_PRESS_MASK|GDK_KEY_PRESS_MASK);
+#if GTKVERSION==2
       gtk_widget_set_events(window, GDK_BUTTON_PRESS_MASK);
-
-    gtk_signal_connect(GTK_OBJECT (window), "button_press_event",
-		       (GtkSignalFunc) zg_button_press_event, NULL);
-
+      gtk_signal_connect(GTK_OBJECT (window), "button_press_event",
+  		       (GtkSignalFunc) zg_button_press_event, NULL);
     gtk_widget_set_events(drw, GDK_EXPOSURE_MASK);
     gtk_signal_connect(GTK_OBJECT (drw), "expose_event",
-		       (GtkSignalFunc) zg_expose_event, NULL);
-
+  		       (GtkSignalFunc) zg_expose_event, NULL);
     gtk_signal_connect(GTK_OBJECT (window), "destroy",
-		       (GtkSignalFunc) zg_destroy_event, NULL);
-
-    gtk_widget_show(window);
+  		       (GtkSignalFunc) zg_destroy_event, NULL);
+#else
+//      gtk_widget_set_events(drw, GDK_EXPOSURE_MASK);
+      gtk_widget_set_events(window, GDK_BUTTON_PRESS_MASK);
+      g_signal_connect(G_OBJECT(window), "key_press_event",
+                          G_CALLBACK(zg_key_press_event), NULL);
+    g_signal_connect (G_OBJECT(drw), "draw",
+                      G_CALLBACK(zg_draw_event), NULL);
+    g_signal_connect(G_OBJECT (window), "destroy",
+  		      G_CALLBACK(zg_destroy_event), NULL);
+#endif
+      gtk_widget_show(window);
   }
-
   return drw;
 }
 
@@ -667,7 +760,7 @@ void ZGDOPN(DCL_INT *dev_type,
     dump    = TRUE;
   }
 
-  cfnchr(dmpfile, file, 79);
+  cfnchr(dmpfile, file, 1023);
   cfnchr(xtitle, title, 79);
 
   /* check drawing_area, if not set, stop */
@@ -761,8 +854,11 @@ void ZGDOPN(DCL_INT *dev_type,
   if (ldclonly) {
 
     cr =zgcsfc();
+#if GTKVERSION==2
     zgupdate(0);
-
+#else
+    zgupdate(0,cr);
+#endif
     if (wait_op) {
       next = FALSE;
       ifunc = DOPN;
@@ -884,7 +980,11 @@ void ZGPOPN(void)
      cairo_rectangle(cro,0.0,0.0,wsxwd,wsywd);
      cairo_fill(cro);
      if(!laltz){
-       zgupdate(0);
+#if GTKVERSION==2
+    zgupdate(0);
+#else
+    zgupdate(0,cro);
+#endif
     }
   }
 
@@ -909,7 +1009,13 @@ void ZGPCLS(void)
   char cnum[4];
 
   if (ldclonly) {
-    if(lpage)zgupdate(0);
+    if(lpage){
+#if GTKVERSION==2
+      zgupdate(0);
+#else
+      zgupdate(0,cr);
+#endif
+    }
     if (wait_np) {
       dumpz = FALSE;
       next = FALSE;
@@ -998,7 +1104,12 @@ void ZGFLASH()
     irxb=0;
     iryt=0;
     iryb=0;
+#if GTKVERSION==2
     zgupdate(0);
+#else
+    zgupdate(0,cr);
+#endif
+
 }
 
 /*------------------------- object ------------------------*/
@@ -1018,7 +1129,14 @@ void zgocls_(char *objname)
 void ZGOCLS(char *objname)
 #endif
 {
-  if (!laltz) zgupdate(1);
+  if (!laltz){
+#if GTKVERSION==2
+    zgupdate(1);
+#else
+    zgupdate(1,cr);
+#endif
+
+}
 
   /* for long-type message? */
 }
@@ -1051,7 +1169,11 @@ void ZGSCLI(DCL_INT *iclidx)
 
   if (!lfcmod){
 
-  iclidz = *iclidx % MAXCLI;
+  if (*iclidx == ibgcli) {
+    iclidz=0;
+  }else{
+    iclidz = *iclidx % MAXCLI;
+  }
 
   if(*iclidx==ifcidx){
     rt1=rl2;
@@ -1194,9 +1316,12 @@ void dcltext2pangomarkup(char* text, int textlength, char* string, double fontsi
 
   sg_small = getrparm("small");
   sg_shift = getrparm("shift");
-  sg_isup = getiparm("isup");
+/*  sg_isup = getiparm("isup");
   sg_isub = getiparm("isub");
-  sg_irst = getiparm("irst");
+  sg_irst = getiparm("irst");*/
+  sg_isup = 29;
+  sg_isub = 30;
+  sg_irst = 31;
   sg_lcntl = getlparm("lcntl");
 
 /* calculate rise amount for super-/sub-script from fontsize that in PangoUnits
@@ -1558,13 +1683,7 @@ void dcltext2pangomarkup(char* text, int textlength, char* string, double fontsi
 
     if (0){ /* 従来のDCL書式 */
       /* 制御文字をmarkup tagに。 */
-      if (dclextchar[i] == sg_isup && sg_lcntl){ /* 上付き開始 */
-        strcpy(&string[length], sup_span); length += strlen(sup_span); continue;
-      }else if(dclextchar[i] == sg_isub && sg_lcntl){ /* 下付き開始 */
-        strcpy(&string[length], sub_span); length += strlen(sub_span); continue;
-      }else if(dclextchar[i] == sg_irst && sg_lcntl){ /* span終了 */
-        strcpy(&string[length], "</span>"); length += strlen("</span>"); continue;
-      }else if(dclextchar[i] == 36 && sg_lcntl){ /* 暫定 $ で over を表現 */
+/*      else if(dclextchar[i] == 36){ // 暫定 $ で over を表現
         sw = char_width(" ", -1);
         cw0 = char_width(&dclextchar[prev_pos], prev_byte);
         cw1 = char_width(&dclextchar[i+1], utf8_char_byte(&dclextchar[i+1]));
@@ -1573,11 +1692,18 @@ void dcltext2pangomarkup(char* text, int textlength, char* string, double fontsi
         height = char_height(&dclextchar[prev_pos], prev_byte);
         sprintf(over_span,"<span letter_spacing='%d'> </span><span rise='%d' letter_spacing='%d'>",
                 -(cw0+sw), (int)(height*0.8), wdiff);
-        strcpy(&string[length], over_span); length += strlen(over_span); continue;
-      }
+        strcpy(&string[length], over_span); length += strlen(over_span); continue;*/
+
 
 
     }else{ /* 新しい書式 */
+      if (dclextchar[i] == sg_isup){ /* 上付き開始 */
+        strcpy(&string[length], sup_span); length += strlen(sup_span); continue;
+      }else if(dclextchar[i] == sg_isub){ /* 下付き開始 */
+        strcpy(&string[length], sub_span); length += strlen(sub_span); continue;
+      }else if(dclextchar[i] == sg_irst){ /* span終了 */
+        strcpy(&string[length], "</span>"); length += strlen("</span>"); continue;
+      }
       if (dclextchar[i] == 92){ /* 92 は バックスラッシュ（制御の始まり） */
         flag_inspan += 1;
         if (flag_close == i){ /* } がない場合の制御終了処理 */
@@ -1747,7 +1873,6 @@ void dcltext2pangomarkup(char* text, int textlength, char* string, double fontsi
   if (flag_close == textlength){
     strcpy(&string[length], "</span>"); length += strlen("</span>");
   }
-
 }
 
 
@@ -1778,14 +1903,13 @@ void zgqtxw_(char* text, DCL_INT *len, DCL_REAL *wxch, DCL_REAL *wych)
 void ZGQTXW(char* text, DCL_INT *len, DCL_REAL *wxch, DCL_REAL *wych)
 #endif
 {
-  char string[1024]={0} ;
+  char string[32768]={0} ;
   double width, height;
   double const fact=100.0;
 
   dcltext2pangomarkup(text, *len, string, 1.0*PANGO_SCALE*fact);
   pango_font_description_set_size(font_desc, 1.0*PANGO_SCALE*fact);
   get_pangostring_width_height(cr, string, &width, &height);
-
   *wxch = width*1.0/fact; //目で見て調整しただけ。
   *wych = height*0.8/fact;
 }
@@ -1827,7 +1951,7 @@ void ZGTXT(DCL_REAL *wx, DCL_REAL *wy, DCL_REAL *size, char* text, DCL_INT *len,
 {
   DCL_REAL iiwx,iiwy;
   double font_size;
-  char string[2048]={0};
+  char string[32768]={0};
   int char_byte;
   double rotation;
   double width, height;
@@ -1835,7 +1959,6 @@ void ZGTXT(DCL_REAL *wx, DCL_REAL *wy, DCL_REAL *size, char* text, DCL_INT *len,
 
   /* set font size and face. calculate string's width and height */
   font_size = *size * FSFACT;
-  //pango_font_description_set_absolute_size(font_desc, font_size*PANGO_SCALE);
   pango_font_description_set_size(font_desc, (gint)font_size*PANGO_SCALE);
 
   /* convert dcl's text to pango's markup */
@@ -1892,14 +2015,25 @@ void ZGTXT(DCL_REAL *wx, DCL_REAL *wy, DCL_REAL *size, char* text, DCL_INT *len,
   zguprect(ltx,lty);
   zguprect(rbx,rby);
   zguprect(rtx,rty);
-  if(!laltz){zgupdate(1);}
+
+  if(!laltz){
+#if GTKVERSION==2
+    zgupdate(1);
+#else
+    zgupdate(1,cr);
+#endif
+  }
 }
 
 
 
 
 static void font_dialog_response(
+#if GTKVERSION==2
            GtkFontSelectionDialog *dialog,
+#else
+           GtkFontChooser *dialog,
+#endif
            gint response, gpointer data){
 
   gchar *font;
@@ -1907,7 +2041,12 @@ static void font_dialog_response(
   switch (response){
   case (GTK_RESPONSE_APPLY):
   case (GTK_RESPONSE_OK):
+#if GTKVERSION==2
     font=gtk_font_selection_dialog_get_font_name(dialog);
+#else
+    font=gtk_font_chooser_get_font(dialog);
+
+#endif
     font_desc = pango_font_description_from_string (font);
     printf("selected font name is  '%s'  \n",font);
     strcpy(fontfamily, font);
@@ -1931,23 +2070,41 @@ void zgselectfont_(char* selected_font){
 #else
 void ZGSELECTFONT(char* selected_font){
 #endif
+  int i;
   GtkWidget *dialog;
 //  static void font_dialog_response (GtkFontSelectionDialog*, gint, gpointer);
 
+#if GTKVERSION==2
   dialog = gtk_font_selection_dialog_new("Choose a Font");
+#else
+  dialog = gtk_font_chooser_dialog_new("Choose a Font",(GtkWindow*)window);
+#endif
   if (fontfamily == NULL){
     fontfamily = defaultfont;
   }
+#if GTKVERSION==2
   gtk_font_selection_dialog_set_font_name(
            GTK_FONT_SELECTION_DIALOG(dialog),
            fontfamily);
   gtk_font_selection_dialog_set_preview_text(
            GTK_FONT_SELECTION_DIALOG(dialog),
            "GFD Dennou Common Library 電脳ライブラリ αβΣΩ");
+#else
+  gtk_font_chooser_set_font(
+           GTK_FONT_CHOOSER(dialog),
+           fontfamily);
+  gtk_font_chooser_set_preview_text(
+           GTK_FONT_CHOOSER(dialog),
+           "GFD Dennou Common Library 電脳ライブラリ αβΣΩ");
+#endif
+
   g_signal_connect(G_OBJECT(dialog),"response",
            G_CALLBACK(font_dialog_response),NULL);
   gtk_dialog_run(GTK_DIALOG(dialog));
 
+  for(i=0;i<1024;i++){
+    selected_font[i]=0;
+  }
   strcpy(selected_font, fontfamily);
 
 }
@@ -2131,8 +2288,11 @@ void ZGGTON(DCL_INT *np, DCL_REAL wpx[], DCL_REAL wpy[], DCL_INT *itpat)
   }
 
   //  ibit = *itpat % 1000;
-  iclr = (*itpat / 1000) % MAXCLI;
-
+  if (*itpat/1000 == ibgcli) {
+    iclr=0;
+  }else{
+    iclr = (*itpat / 1000) % MAXCLI;
+  }
   if ((*itpat / 1000) == ifcidx){
     rt1=rt2;
     gt1=gt2;
@@ -2144,8 +2304,8 @@ void ZGGTON(DCL_INT *np, DCL_REAL wpx[], DCL_REAL wpy[], DCL_INT *itpat)
   }
 
   cairo_set_source_rgb(crb,rt1,gt1,bt1);
-  cairo_fill(crb);
   cairo_rectangle(crb,0.0,0.0,16.0,16.0);
+  cairo_fill(crb);
 
   pattern = cairo_pattern_create_for_surface (csb);
   cairo_pattern_set_extend (pattern , CAIRO_EXTEND_REPEAT);
@@ -2169,9 +2329,7 @@ void ZGGTON(DCL_INT *np, DCL_REAL wpx[], DCL_REAL wpy[], DCL_INT *itpat)
   }
 
   cairo_close_path(cr);
-
   cairo_set_source_rgb(cr,rt1,gt1,bt1);
-
 // Adhoc Patch for 3
   if ( idev_type == IWS_FILE && ifl == IFL_PDF ){
     cairo_fill_preserve(cr);
@@ -2267,7 +2425,11 @@ void ZGICLS(void)
     irxb=0;
     iryt=0;
     iryb=0;
+#if GTKVERSION==2
     zgupdate(0);
+#else
+    zgupdate(0,cr);
+#endif
   }
   fprintf (stderr, " end\n");
   cairo_surface_destroy(csi);
@@ -2422,12 +2584,13 @@ void zgstcl_(DCL_INT *icolor)
 void ZGSTCL(DCL_INT *icolor)
 #endif
 {
+  /*This Function is not used in uipk samples
+     So, it is not tested.*/
   /* set tone color in 24 bit RGB */
   GdkColor  loccx;
+  double rt1,gt1,bt1;
 
   if (ltfrst) {
-//    gct = gdk_gc_new (drawing_area->window);
-
     ltfrst = FALSE;
   }
 
@@ -2439,6 +2602,7 @@ void ZGSTCL(DCL_INT *icolor)
   gt2 = (double) loccx.green / 65535;
   bt2 = (double) loccx.blue  / 65535;
 
+  cairo_set_source_rgb(cr,rt1,gt1,bt1);
 }
 
 #ifndef WINDOWS
@@ -2483,7 +2647,13 @@ void ZGICLR(DCL_INT *image,DCL_INT *nlen)
       }
     }
   }
-  if(!laltz){zgupdate(1);}
+  if(!laltz){
+#if GTKVERSION==2
+    zgupdate(1);
+#else
+    zgupdate(1,cr);
+#endif
+  }
 }
 
 #ifndef WINDOWS
